@@ -3,12 +3,11 @@ import random
 
 import async_timeout
 import pytest
-from async_generator import async_generator, yield_
 
 from asgiref.sync import async_to_sync
-from channels_redis.core import ChannelFull, ConnectionPool, RedisChannelLayer
+from channels_redis.core import ChannelFull, RedisChannelLayer
 
-TEST_HOSTS = [("localhost", 6379)]
+TEST_HOSTS = ["redis://localhost:6379"]
 
 MULTIPLE_TEST_HOSTS = [
     "redis://localhost:6379/0",
@@ -55,7 +54,6 @@ async def group_send_three_messages_with_delay(group_name, channel_layer, delay)
 
 
 @pytest.fixture()
-@async_generator
 async def channel_layer():
     """
     Channel layer fixture that flushes automatically.
@@ -63,18 +61,17 @@ async def channel_layer():
     channel_layer = RedisChannelLayer(
         hosts=TEST_HOSTS, capacity=3, channel_capacity={"tiny": 1}
     )
-    await yield_(channel_layer)
+    yield channel_layer
     await channel_layer.flush()
 
 
 @pytest.fixture()
-@async_generator
 async def channel_layer_multiple_hosts():
     """
     Channel layer fixture that flushes automatically.
     """
     channel_layer = RedisChannelLayer(hosts=MULTIPLE_TEST_HOSTS, capacity=3)
-    await yield_(channel_layer)
+    yield channel_layer
     await channel_layer.flush()
 
 
@@ -404,31 +401,22 @@ async def test_group_send_capacity_multiple_channels(channel_layer, caplog):
         )
 
 
-@pytest.mark.asyncio
-async def test_connection_pool_pop():
+def test_repeated_group_send_with_async_to_sync(channel_layer):
     """
-    Makes sure that the connection pool does not return closed connections
+    Makes sure repeated group_send calls wrapped in async_to_sync
+    process-local channel names.
     """
+    channel_layer = RedisChannelLayer(hosts=TEST_HOSTS, capacity=3)
 
-    # Setup scenario
-    connection_pool = ConnectionPool({"address": TEST_HOSTS[0]})
-    conn = await connection_pool.pop()
-
-    # Emualte a disconnect and return it to the pool
-    conn.close()
-    assert conn.closed
-    connection_pool.push(conn)
-
-    # Ensure the closed connection is inside the pool
-    conn_map_values = list(connection_pool.conn_map.values())
-    assert len(conn_map_values) == 1
-    conns = conn_map_values[0]
-    assert len(conns) == 1
-    assert conns[0].closed
-
-    # Retrieve new connection
-    conn = await connection_pool.pop()
-    assert not conn.closed
+    try:
+        async_to_sync(channel_layer.group_send)(
+            "channel_name_1", {"type": "test.message.1"}
+        )
+        async_to_sync(channel_layer.group_send)(
+            "channel_name_2", {"type": "test.message.2"}
+        )
+    except RuntimeError as exc:
+        pytest.fail(f"repeated async_to_sync wrapped group_send calls raised {exc}")
 
 
 @pytest.mark.asyncio
@@ -436,7 +424,7 @@ async def test_receive_cancel(channel_layer):
     """
     Makes sure we can cancel a receive without blocking
     """
-    channel_layer = RedisChannelLayer(capacity=10)
+    channel_layer = RedisChannelLayer(capacity=30)
     channel = await channel_layer.new_channel()
     delay = 0
     while delay < 0.01:
@@ -641,3 +629,26 @@ def test_receive_buffer_respects_capacity():
     assert buff.qsize() == capacity
     messages = [buff.get_nowait() for _ in range(capacity)]
     assert list(range(9900, 10000)) == messages
+
+
+def test_serialize():
+    """
+    Test default serialization method
+    """
+    message = {"a": True, "b": None, "c": {"d": []}}
+    channel_layer = RedisChannelLayer()
+    serialized = channel_layer.serialize(message)
+    assert isinstance(serialized, bytes)
+    assert serialized[12:] == b"\x83\xa1a\xc3\xa1b\xc0\xa1c\x81\xa1d\x90"
+
+
+def test_deserialize():
+    """
+    Test default deserialization method
+    """
+    message = b"Q\x0c\xbb?Q\xbc\xe3|D\xfd9\x00\x83\xa1a\xc3\xa1b\xc0\xa1c\x81\xa1d\x90"
+    channel_layer = RedisChannelLayer()
+    deserialized = channel_layer.deserialize(message)
+
+    assert isinstance(deserialized, dict)
+    assert deserialized == {"a": True, "b": None, "c": {"d": []}}
