@@ -3,25 +3,81 @@ import random
 
 import async_timeout
 import pytest
-from async_generator import async_generator, yield_
 
 from asgiref.sync import async_to_sync
-from channels_redis.core import ChannelFull, ConnectionPool, RedisChannelLayer
+from channels_redis.core import ChannelFull, RedisChannelLayer
 
 SENTINEL_MASTER = "sentinel"
-TEST_HOSTS = [{"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER}]
+SENTINEL_KWARGS = {"password": "channels_redis"}
 
+TEST_HOSTS = [
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+    }
+]
 MULTIPLE_TEST_HOSTS = [
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 0},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 1},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 2},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 3},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 4},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 5},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 6},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 7},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 8},
-    {"sentinels": [("localhost", 26379)], "master_name": SENTINEL_MASTER, "db": 9},
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 0,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 1,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 2,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 3,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 4,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 5,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 6,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 7,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 8,
+    },
+    {
+        "sentinels": [("localhost", 26379)],
+        "master_name": SENTINEL_MASTER,
+        "sentinel_kwargs": SENTINEL_KWARGS,
+        "db": 9,
+    },
 ]
 
 
@@ -56,28 +112,24 @@ async def group_send_three_messages_with_delay(group_name, channel_layer, delay)
 
 
 @pytest.fixture()
-@async_generator
 async def channel_layer():
     """
     Channel layer fixture that flushes automatically.
     """
     channel_layer = RedisChannelLayer(
-        hosts=TEST_HOSTS,
-        capacity=3,
-        channel_capacity={"tiny": 1},
+        hosts=TEST_HOSTS, capacity=3, channel_capacity={"tiny": 1}
     )
-    await yield_(channel_layer)
+    yield channel_layer
     await channel_layer.flush()
 
 
 @pytest.fixture()
-@async_generator
 async def channel_layer_multiple_hosts():
     """
     Channel layer fixture that flushes automatically.
     """
     channel_layer = RedisChannelLayer(hosts=MULTIPLE_TEST_HOSTS, capacity=3)
-    await yield_(channel_layer)
+    yield channel_layer
     await channel_layer.flush()
 
 
@@ -409,39 +461,12 @@ async def test_group_send_capacity_multiple_channels(channel_layer, caplog):
         )
 
 
-@pytest.mark.asyncio
-async def test_connection_pool_pop():
-    """
-    Makes sure that the connection pool does not return closed connections
-    """
-
-    # Setup scenario
-    connection_pool = ConnectionPool(TEST_HOSTS[0])
-    conn = await connection_pool.pop()
-
-    # Emualte a disconnect and return it to the pool
-    conn.close()
-    assert conn.closed
-    connection_pool.push(conn)
-
-    # Ensure the closed connection is inside the pool
-    conn_map_values = list(connection_pool.conn_map.values())
-    assert len(conn_map_values) == 1
-    conns = conn_map_values[0]
-    assert len(conns) == 1
-    assert conns[0].closed
-
-    # Retrieve new connection
-    conn = await connection_pool.pop()
-    assert not conn.closed
-
-
 @pytest.mark.asyncio
 async def test_receive_cancel(channel_layer):
     """
     Makes sure we can cancel a receive without blocking
     """
-    channel_layer = RedisChannelLayer(capacity=10)
+    channel_layer = RedisChannelLayer(capacity=30)
     channel = await channel_layer.new_channel()
     delay = 0
     while delay < 0.01:
@@ -646,3 +671,26 @@ def test_receive_buffer_respects_capacity():
     assert buff.qsize() == capacity
     messages = [buff.get_nowait() for _ in range(capacity)]
     assert list(range(9900, 10000)) == messages
+
+
+def test_serialize():
+    """
+    Test default serialization method
+    """
+    message = {"a": True, "b": None, "c": {"d": []}}
+    channel_layer = RedisChannelLayer()
+    serialized = channel_layer.serialize(message)
+    assert isinstance(serialized, bytes)
+    assert serialized[12:] == b"\x83\xa1a\xc3\xa1b\xc0\xa1c\x81\xa1d\x90"
+
+
+def test_deserialize():
+    """
+    Test default deserialization method
+    """
+    message = b"Q\x0c\xbb?Q\xbc\xe3|D\xfd9\x00\x83\xa1a\xc3\xa1b\xc0\xa1c\x81\xa1d\x90"
+    channel_layer = RedisChannelLayer()
+    deserialized = channel_layer.deserialize(message)
+
+    assert isinstance(deserialized, dict)
+    assert deserialized == {"a": True, "b": None, "c": {"d": []}}
